# Summary

- [Introduction](README.md)
  - [Resources](docs/intro/README.md#resources)
  - [Questions?](docs/intro/README.md#questions)
  - [Terminology](docs/intro/README.md#terminology)
  - [Customised built-in elements](docs/intro/README.md#customised-built-in-elements)
  - [VS other libraries](docs/intro/README.md#vs-other-libraries)
    - [VS WebComponentsJS](docs/intro/README.md#vs-webcomponentsjs)
    - [VS Polymer](docs/intro/README.md#vs-polymer)
    - [VS X-Tags](docs/intro/README.md#vs-x-tags)
    - [VS React](docs/intro/README.md#vs-react)

----

- [Examples](docs/examples/README.md)
  - [Counter](docs/examples/README.md#counter)

----

- [Recipes](docs/recipes/README.md)
  - [Designing Web Components](docs/recipes/designing-web-components.md)
    - [Imperative](docs/recipes/designing-web-components.md#imperative)
    - [Declarative](docs/recipes/designing-web-components.md#declarative)
    - [Naming Collisions](docs/recipes/designing-web-components.md#naming-collisions)
    - [Compatible with multiple versions of itself](docs/recipes/designing-web-components.md#compatible-with-multiple-versions-of-itself)
    - [Properties and Attributes](docs/recipes/designing-web-components.md#properties-and-attributes)
    - [Private Members](docs/recipes/designing-web-components.md#private-members)
    - [Private Data](docs/recipes/designing-web-components.md#private-data)
  - [Design patterns](docs/recipes/design-patterns/README.md)
    - [Composition/Extending](docs/recipes/design-patterns/component-composition.md)
  - [Framework Integration](docs/recipes/framework-integration/README.md)
    - [React](docs/recipes/framework-integration/React.md)
    - [Preact](docs/recipes/framework-integration/Preact.md)
    - [Angular 2](docs/recipes/framework-integration/Angular-2.md)
  - [Form Behaviour and the Shadow DOM](docs/recipes/form-behaviour-and-the-shadow-dom.md)
    - [Submission](docs/recipes/form-behaviour-and-the-shadow-dom.md#submission)
    - [Form Data](docs/recipes/form-behaviour-and-the-shadow-dom.md#form-data)
  - [Stateless Components](docs/recipes/stateless-components.md)
  - [Styling Components](docs/recipes/styling-components.md)
  - [Preventing FOUC](docs/recipes/preventing-FOUC.md)

----

- [API](docs/api/README.md)
  - [Component](docs/api/Component.md)
    - [`constructor`](docs/api/Component.md#constructor---supersedes-static-created)
    - [`connectedCallback`](docs/api/Component.md#connectedcallback---supersedes-static-attached)
    - [`updatedCallback`](docs/api/Component.md#updatedcallback---supersedes-static-updated)
      - [Other use-cases](docs/api/Component.md#other-use-cases)
    - [`renderCallback`](docs/api/Component.md#rendercallback---supersedes-static-render)
      - [Return Value](docs/api/Component.md#return-value)
    - [`renderedCallback`](docs/api/Component.md#renderedcallback---supersedes-static-rendered)
    - [`attributeChangedCallback`](docs/api/Component.md#attributechangedcallback---supersedes-static-attributechanged)
    - [`disconnectedCallback`](docs/api/Component.md#disconnectedcallback---supersedes-static-detached)
    - [`prototype`](docs/api/Component.md#prototype)
    - [`static observedAttributes`](docs/api/Component.md#static-observedattributes)
    - [`static props`](docs/api/Component.md#static-props)
      - [`attribute`](docs/api/Component.md#attribute)
      - [`coerce`](docs/api/Component.md#coerce)
      - [`default`](docs/api/Component.md#default)
      - [`deserialize`](docs/api/Component.md#deserialize)
      - [`get`](docs/api/Component.md#get)
      - [`initial`](docs/api/Component.md#initial)
      - [`serialize`](docs/api/Component.md#serialize)
      - [`set`](docs/api/Component.md#set)
  - [`define (nameOrConstructor, Constructor)`](docs/api/define.md)
    - [WebPack Hot-Module Reloading](docs/api/define.md#webpack-hot-module-reloading)
  - [`emit (elem, eventName, eventOptions = {})`](docs/api/emit.md)
    - [Preventing Bubbling or Canceling](docs/api/emit.md#preventing-bubbling-or-canceling)
    - [Passing Data](docs/api/emit.md#passing-data)
  - [`link (elem, propSpec)`](docs/api/link.md)
  - [`prop`](docs/api/prop.md)
    - [`array`](docs/api/prop.md#array)
    - [`boolean`](docs/api/prop.md#boolean)
    - [`number`](docs/api/prop.md#number)
    - [`string`](docs/api/prop.md#string)
  - [`props (elem[, props])`](docs/api/props.md)
  - [`ready (element, callback)`](docs/api/ready.md)
    - [Background](docs/api/ready.md#background)
  - [`h`](docs/api/h.md)
    - [`Hyperscript`](docs/api/h.md#hyperscript)
    - [`JSX`](docs/api/h.md#jsx)
      - [Setting `pragma`](docs/api/h.md#setting-pragma)
      - [Default `React.createElement()`](docs/api/h.md#default-reactcreateelement)
      - [Other ways to use JSX](docs/api/h.md#other-ways-to-use-jsx)
  - [`vdom`](docs/api/vdom.md)
    - [`vdom.builder ()`](docs/api/vdom.md#vdombuilder-)
    - [`vdom.builder (...elements)`](docs/api/vdom.md#vdombuilder-elements)
    - [[DEPRECATED] `vdom.element (elementName, attributesOrChildren, ...children)`](docs/api/vdom.md#deprecated-vdomelement-elementname-attributesorchildren-children)
    - [[DEPRECATED] `vdom.text (text)`](docs/api/vdom.md#deprecated-vdomtext-text)
    - [Incremental DOM](docs/api/vdom.md#incremental-dom)
      - [Component constructor](docs/api/vdom.md#component-constructor)
      - [Function helper](docs/api/vdom.md#function-helper)
      - [Special Attributes](docs/api/vdom.md#special-attributes)
        - [`class`](docs/api/vdom.md#class)
        - [`key`](docs/api/vdom.md#key)
        - [`on*`](docs/api/vdom.md#on)
        - [`ref`](docs/api/vdom.md#ref)
        - [`skip`](docs/api/vdom.md#skip)
        - [`statics`](docs/api/vdom.md#statics)
        - [Boolean Attributes](docs/api/vdom.md#boolean-attributes)

---

- [Change Log](CHANGELOG.md)
